@@ -1,7 +1,8 @@
 from bespin.amazon.s3 import delete_key_from_s3, list_keys_from_s3_path, upload_file_to_s3, upload_file_to_s3_as_single
 from bespin.amazon.ec2 import get_instances_in_asg_by_lifecycle_state
+from bespin.amazon.sqs import get_all_deployment_messages
 from bespin.option_spec import stack_specs
-from bespin.errors import NoSuchStack
+from bespin.errors import NoSuchStack, BadDeployment
 from bespin.layers import Layers
 from bespin import helpers as hp
 
@@ -59,6 +60,10 @@ class Builder(object):
         self.build_stack(stack)
         made[stack.name] = True
 
+        if stack.sns_confirmation is not NotSpecified and stack.sns_confirmation.straight_after:
+            stack.cloudformation.wait()
+            self.confirm_deployment(stack, credentials)
+
         if any(stack.build_after):
             stack.cloudformation.wait()
             for dependency in stack.build_after:
@@ -68,6 +73,8 @@ class Builder(object):
                 stacks[dependency].cloudformation.wait()
 
         stack.cloudformation.wait()
+        if stack.sns_confirmation is not NotSpecified and not stack.sns_confirmation.straight_after:
+            self.confirm_deployment(stack, credentials)
 
     def layered(self, stacks, only_pushable=False):
         """Yield layers of stacks"""
@@ -104,7 +111,7 @@ class Builder(object):
             # Create a temporary file to tar to
             with hp.a_temp_file() as temp_tar_file:
                 # Make the artifact
-                hp.generate_tar_file(temp_tar_file, artifact.paths + artifact.files
+                hp.generate_tar_file(temp_tar_file, artifact.commands + artifact.paths + artifact.files
                     , environment=environment
                     , compression=artifact.compression_type
                     )
@@ -114,20 +121,49 @@ class Builder(object):
                 upload_file_to_s3_as_single(credentials, temp_tar_file.name, artifact.upload_to.format(**environment))
 
     def confirm_deployment(self, stack, credentials):
-        # Find missing env before doing anything
-        self.find_missing_build_env(stack)
+        autoscaling_group_id = stack.sns_confirmation.autoscaling_group_id
+        asg_physical_id = stack.cloudformation.map_logical_to_physical_resource_id(autoscaling_group_id)
+        instances_to_check = get_instances_in_asg_by_lifecycle_state(credentials, asg_physical_id, lifecycle_state="InService")
 
-        asg_physical_id = stack.cloudformation.map_logical_to_physical_resource_id("AppServerAutoScalingGroup")
-        instances = get_instances_in_asg_by_lifecycle_state(credentials, asg_physical_id, lifecycle_state="InService")
+        environment = dict(env.pair for env in stack.sns_confirmation.env)
+        version_message = stack.sns_confirmation.version_message.format(**environment)
 
-        for instance in instances:
-            print(instance)
+        failed = []
+        success = []
+        attempt = 0
 
-        # Iterate over each artifact we need to clean
-        for key, artifact in stack.artifacts.items():
-            environment = dict(env.pair for env in artifact.build_env)
-            version_message = artifact.version_message.format(**environment)
-            log.info("All stacks have been confirmed to be deployed with version_message [%s]!", version_message)
+        for _ in hp.until(action="Printing instance list"):
+            messages = get_all_deployment_messages(credentials, stack.sns_confirmation.deployment_queue)
+
+            # Look for success and failure in the messages
+            for message in messages:
+                log.info("Message received %s", message['output'])
+
+                # Ignore the messages for instances outside this deployment
+                if message['instance_id'] in instances_to_check:
+                    if message['output'] == version_message:
+                        log.info("Deployed instance %s", message['instance_id'])
+                        success.append(message['instance_id'])
+                    else:
+                        log.info("Failed to deploy instance %s", message['instance_id'])
+                        log.info("Failure Message: ", "%s", message['output'])
+                        failed.append(message['instance_id'])
+
+            # Stop trying if we have all the instances
+            if set(failed + success) == set(instances_to_check):
+                break
+
+            # Record the iteration of checking for a valid deployment
+            attempt += 1
+            log.info("Completed attempt %s of checking for a valid deployment state", attempt)
+
+        if success:
+            log.info("Succeeded to deploy %s", success)
+        if failed:
+            log.info("Failed to deploy %s", failed)
+            raise BadDeployment("")
+
+        log.info("All instances have been confirmed to be deployed with version_message [%s]!", version_message)
 
     def clean_old_artifacts(self, stack, credentials):
         # Find missing env before doing anything
